@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2006-2021 sqlmap developers (http://sqlmap.org/)
+Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/)
 See the file 'LICENSE' for copying permission
 """
 
@@ -37,7 +37,7 @@ class Fingerprint(GenericFingerprint):
 
         if not result:
             warnMsg = "unable to perform %s comment injection" % DBMS.MYSQL
-            logger.warn(warnMsg)
+            logger.warning(warnMsg)
 
             return None
 
@@ -45,9 +45,9 @@ class Fingerprint(GenericFingerprint):
         # Reference: https://dev.mysql.com/doc/relnotes/mysql/<major>.<minor>/en/
 
         versions = (
-            (80000, 80028),  # MySQL 8.0
+            (80000, 80033),  # MySQL 8.0
             (60000, 60014),  # MySQL 6.0
-            (50700, 50736),  # MySQL 5.7
+            (50700, 50742),  # MySQL 5.7
             (50600, 50652),  # MySQL 5.6
             (50500, 50563),  # MySQL 5.5
             (50400, 50404),  # MySQL 5.4
@@ -166,9 +166,8 @@ class Fingerprint(GenericFingerprint):
         if not conf.extensiveFp and Backend.isDbmsWithin(MYSQL_ALIASES):
             setDbms("%s %s" % (DBMS.MYSQL, Backend.getVersion()))
 
-            if Backend.isVersionGreaterOrEqualThan("5"):
+            if Backend.isVersionGreaterOrEqualThan("5") or inject.checkBooleanExpression("DATABASE() LIKE SCHEMA()"):
                 kb.data.has_information_schema = True
-
             self.getBanner()
 
             return True
@@ -176,7 +175,7 @@ class Fingerprint(GenericFingerprint):
         infoMsg = "testing %s" % DBMS.MYSQL
         logger.info(infoMsg)
 
-        result = inject.checkBooleanExpression("QUARTER(NULL) IS NULL")
+        result = inject.checkBooleanExpression("QUARTER(NULL XOR NULL) IS NULL")
 
         if result:
             infoMsg = "confirming %s" % DBMS.MYSQL
@@ -193,23 +192,23 @@ class Fingerprint(GenericFingerprint):
 
             if not result:
                 warnMsg = "the back-end DBMS is not %s" % DBMS.MYSQL
-                logger.warn(warnMsg)
+                logger.warning(warnMsg)
 
                 return False
 
             # reading information_schema on some platforms is causing annoying timeout exits
             # Reference: http://bugs.mysql.com/bug.php?id=15855
 
+            kb.data.has_information_schema = True
+
             # Determine if it is MySQL >= 8.0.0
             if inject.checkBooleanExpression("ISNULL(JSON_STORAGE_FREE(NULL))"):
-                kb.data.has_information_schema = True
                 Backend.setVersion(">= 8.0.0")
                 setDbms("%s 8" % DBMS.MYSQL)
                 self.getBanner()
 
             # Determine if it is MySQL >= 5.0.0
             elif inject.checkBooleanExpression("ISNULL(TIMESTAMPADD(MINUTE,[RANDNUM],NULL))"):
-                kb.data.has_information_schema = True
                 Backend.setVersion(">= 5.0.0")
                 setDbms("%s 5" % DBMS.MYSQL)
                 self.getBanner()
@@ -269,6 +268,8 @@ class Fingerprint(GenericFingerprint):
                 setDbms("%s 4" % DBMS.MYSQL)
                 self.getBanner()
 
+                kb.data.has_information_schema = False
+
                 if not conf.extensiveFp:
                     return True
 
@@ -291,10 +292,12 @@ class Fingerprint(GenericFingerprint):
                 setDbms("%s 3" % DBMS.MYSQL)
                 self.getBanner()
 
+                kb.data.has_information_schema = False
+
             return True
         else:
             warnMsg = "the back-end DBMS is not %s" % DBMS.MYSQL
-            logger.warn(warnMsg)
+            logger.warning(warnMsg)
 
             return False
 
